@@ -419,18 +419,26 @@ int lvm_pv_params_set_property(pv_create_params_t params, const char *name,
 static int _pv_create(pv_create_params_t params)
 {
 	struct cmd_context *cmd = (struct cmd_context *)params->libh;
+	int rc = 0;
 
-	if (params->pv_p.size) {
-		if (params->pv_p.size % SECTOR_SIZE) {
+	if (params->pv_p.pva.size) {
+		if (params->pv_p.pva.size % SECTOR_SIZE) {
 			log_errno(EINVAL, "Size not a multiple of 512");
 			return -1;
 		}
-		params->pv_p.size = params->pv_p.size >> SECTOR_SHIFT;
+		params->pv_p.pva.size = params->pv_p.pva.size >> SECTOR_SHIFT;
 	}
 
-	if (!pvcreate_single(cmd, params->pv_name, &params->pv_p))
+	if (!lock_vol(cmd, VG_ORPHANS, LCK_VG_WRITE, NULL)) {
+		log_errno(EINVAL, "Can't get lock for orphan PVs");
 		return -1;
-	return 0;
+	}
+
+	if (!(pvcreate_vol(cmd, params->pv_name, &params->pv_p, 1)))
+		rc = -1;
+
+	unlock_vg(cmd, VG_ORPHANS);
+	return rc;
 }
 
 int lvm_pv_create(lvm_t libh, const char *pv_name, uint64_t size)
@@ -440,7 +448,7 @@ int lvm_pv_create(lvm_t libh, const char *pv_name, uint64_t size)
 	struct saved_env e = store_user_env((struct cmd_context *)libh);
 
 	if (_lvm_pv_params_create(libh, pv_name, &pp)) {
-		pp.pv_p.size = size;
+		pp.pv_p.pva.size = size;
 		rc = _pv_create(&pp);
 	}
 
