@@ -6,4 +6,4 @@ export LVM_TEST_LVMLOCKD_TEST=1
 export LVM_TEST_DEVDIR=/dev
 
 # FIXME:dct: add option to allow --test with sanlock
-LVM_TEST_LVMLOCKD_TEST_DLM=1
+export LVM_TEST_LVMLOCKD_TEST_DLM=1
