@@ -7,14 +7,12 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software Foundation,
-# Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 TEST_RAID=raid456
 
 . shell/lvchange-raid.sh
 
-test -e LOCAL_LVMPOLLD && skip
-
 aux raid456_replace_works || skip
 aux have_raid 1 5 2 || skip
 
