@@ -22,8 +22,12 @@ aux lvmconf 'allocation/maximise_cling = 0' \
 
 aux prepare_vg 8
 
+# It's possible small raid arrays do have problems with reporting in-sync.
+# So try bigger size
+RAID_SIZE=64
+
 # RAID5 single replace
-lvcreate --type raid5 -i 2 -l 2 -n $lv1 $vg "$dev1" "$dev2" "$dev3"
+lvcreate --type raid5 -i 2 -L $RAID_SIZE -n $lv1 $vg "$dev1" "$dev2" "$dev3"
 aux wait_for_sync $vg $lv1
 aux disable_dev "$dev3"
 lvconvert -y --repair $vg/$lv1
@@ -33,7 +37,7 @@ vgextend $vg "$dev3"
 lvremove -ff $vg
 
 # RAID6 double replace
-lvcreate --type raid6 -i 3 -l 2 -n $lv1 $vg \
+lvcreate --type raid6 -i 3 -L $RAID_SIZE -n $lv1 $vg \
     "$dev1" "$dev2" "$dev3" "$dev4" "$dev5"
 aux wait_for_sync $vg $lv1
 aux disable_dev "$dev4" "$dev5"
